"""incorporates GUI for pydtnsim."""

__author__ = "Hunter McConnell <hunter.mcconnell@usask.ca"

from argparse import ArgumentParser
from collections import namedtuple
from multiprocessing import Pool
from os import path
from random import randint

import gui
import os
import sys
import csv
import yaml

from pydtnsim import Network, RandomTraffic, CSVTrace


Simulation = namedtuple(
    "Simulation", ["trace", "node_type", "seed", "epoch", "packet_interval"]
)

# Implement nodes (clustering, routing combos) in community.py


# lifted from examples/shed.py
def run_simulation(simulation):
    """Run a simulation."""
    seed = simulation.seed

    csv = path.join(simulation.trace, "contact.csv")
    metadata = path.join(simulation.trace, "metadata.json")
    trace = CSVTrace(csv, metadata=metadata)

    epoch = 7 * 24 * 60 * 60  # 7 days    how often clusters are recalculated

    node_type = simulation.node_type
    node_options = {
        "tick_rate": 5 * 60,  # 5 mins    duty cycle length
        "epoch": epoch,
        "k": 3,  # 3         k value for k means clustering
        "context": {},
    }
    nodes = {
        node_id: simulation.node_type(**node_options)
        for node_id in range(trace.nodes)
    }

    traffic_options = {
        "seed": seed,
        "start": epoch,
        "step": simulation.packet_interval,  # how often packets are created
    }
    # see init for other traffic options
    traffic = RandomTraffic(nodes, **traffic_options)

    network = Network(nodes, traffic=traffic, trace=trace)
    network.run()

    stats = {
        "trace": simulation.trace,
        "node_type": node_type.__name__,
        "seed": seed,
    }
    stats.update(network.stats_summary)

    return stats


def main(args):
    """Put up the GUI, or reads from config, then runs the sims."""
    if args["no_gui"]:
        with open(args["config"], "r", newline="") as f:
            config = yaml.load(f, Loader=yaml.FullLoader)
    else:
        # put up GUI for user input
        config = gui.main(args)

    # default values so the sim doesn't immediately break
    if "output" not in config:
        config["output"] = "results"
    if "batch" not in config:
        config["batch"] = 1
    if "seeds" not in config or not config["seeds"]:  # seeds dne or is empty
        config["seeds"] = [None]

    # can't run a sim with no nodes
    if "nodes" not in config:
        print("Sim requires at least one node choice")
        quit()

    pool = Pool()
    simulations = []

    if config["batch"] > 1:  # batch mode with random seeds
        for _ in range(config["batch"]):
            seed = randint(0, 500)
            for node_type in config["nodes"]:
                sim = Simulation(
                    trace=config["trace"],
                    node_type=node_type,
                    seed=seed,
                    epoch=config["epoch"],
                    packet_interval=config["packet_interval"],
                )
                simulations.append(sim)
    else:
        for seed in config["seeds"]:  # seed mode with inputted seeds
            for node_type in config["nodes"]:
                sim = Simulation(
                    trace=config["trace"],
                    node_type=node_type,
                    seed=seed,
                    epoch=config["epoch"],
                    packet_interval=config["packet_interval"],
                )
                simulations.append(sim)

    results = {}

    # sorts stats by node type
    for stats in pool.imap_unordered(run_simulation, simulations):
        type = stats["node_type"]
        if type not in results:
            results[type] = []
        results[type].append(stats)

    # create results dir if it does not exist
    dir_path = path.join("..", "results")
    if not os.path.exists(dir_path):
        os.mkdir(dir_path)

    # find unused directory
    dir_path = path.join(dir_path, config["output"])
    if os.path.exists(dir_path):
        i = 0
        while os.path.exists(f"{dir_path}{i}"):
            i += 1
        config["output"] = f"{config['output']}{i}"
        dir_path += str(i)
    os.mkdir(dir_path)

    # dump sim stats in csv
    result_path = path.join(dir_path, "results.csv")
    with open(result_path, "w", newline="") as results_file:
        for node_type in results:
            fieldnames = results[node_type][0].keys()
            writer = csv.DictWriter(results_file, fieldnames=fieldnames)
            writer.writeheader()
            for result in results[node_type]:
                writer.writerow(result)

    # save the config file too
    config_path = path.join(dir_path, "config.yaml")
    with open(config_path, "w", newline="") as f:
        yaml.dump(config, f)

    print("Sim finished, see", dir_path, "for results")


def parse_args(args):
    """Parse arguments."""
    parser = ArgumentParser()
    parser.add_argument(
        "--no_gui",
        "-n",
        action="store_true",
        help="skip gui and read from config file provided",
    )
    parser.add_argument(
        "--config",
        "-c",
        help="config file to read from",
        default="config.yaml",
    )
    args = parser.parse_args(args)
    return vars(args)


if __name__ == "__main__":
    sys.exit(main(parse_args(sys.argv[1:])))
