"""GUI module for pydtnsim."""

__author__ = "Hunter McConnell <hunter.mcconnell@usask.ca>"

import sys
import tkinter as tk
import yaml

from yaml.loader import Loader
from tkinter import BOTH, DISABLED, NSEW, TRUE, W, X, filedialog as fd

from pydtnsim import Node, EpidemicNode
from pydtnsim.community import BubbleKCliqueNode, BubbleLouvainNode
from pydtnsim.community import HCBFKCliqueNode, HCBFLouvainNode

# update here for options to appear in gui
NODES = [
            Node,  # direct delivery
            EpidemicNode,
            BubbleKCliqueNode,
            HCBFKCliqueNode,
            BubbleLouvainNode,
            HCBFLouvainNode,
        ]

INDEPENDANT = [
    "None",
    "contact_dir",
    "epoch",
    "k",
    "buffer_capacity",
    "bandwidth",
    "step",
    "traffic_per_step",
    "time_to_live",
    "payload",
]

DEPENDANT = [
    "delivery-ratio",
    "delivery-cost",
    "latency",
]

class SimOptions(tk.LabelFrame):
    def __init__(self, parent=None):
        tk.LabelFrame.__init__(self, parent, text="SimOptions")
        self.parent = parent

        tk.Label(self, text="Config File:").grid(row=0, column=0)
        self. config_var = tk.StringVar()
        tk.Entry(self, textvariable=self. config_var).grid(row=0, column=1)

        tk.Label(self, text="Contact dir:").grid(row=1, column=0)
        self.trace_var = tk.StringVar()
        tk.Entry(self, textvariable=self.trace_var).grid(row=1, column=1)

        # output file
        # todo: file dialog?
        # todo: validation?
        tk.Label(self, text="Output dir:").grid(row=2, column=0)
        self.out_var = tk.StringVar()
        tk.Entry(self, textvariable=self.out_var).grid(row=2, column=1)

        # seeds
        tk.Label(self, text="Seeds:").grid(row=3, column=0)
        self.seeds_var = tk.StringVar()
        tk.Entry(self, textvariable=self.seeds_var).grid(row=3, column=1)
    
    def getOptions(self):
        """Return a   dictionary of sim options"""
        param = {}
        param["config"] = self.config_var.get()
        param["contact_dir"] = self.trace_var.get()
        param["output_dir"] = self.out_var.get()
        # parse string into list
        param["seeds"] = [
            int(seed.strip())
            for seed in self.seeds_var.get().split(",")
            if seed != ""
        ]

        return param

    def setOptions(self, options):
        self.config_var.set(options["config"])
        self.trace_var.set(options["contact_dir"])
        self.out_var.set(options["output_dir"])
        
        tmp = ", ".join(str(i) for i in options["seeds"])
        self.seeds_var.set(tmp)


class NodeOptions(tk.LabelFrame):
    def __init__(self, parent):
        tk.LabelFrame.__init__(self, parent, text="NodeOptions")
        self.parent = parent

        tk.Label(self, text="Epoch (s):").grid(row=0, column=0)
        self.epoch_var = tk.IntVar(value=604800)
        tk.Entry(self, textvariable=self.epoch_var).grid(row=0, column=1)

        tk.Label(self, text="k:").grid(row=1, column=0)
        self.k_var = tk.IntVar(value=3)
        tk.Entry(self, textvariable=self.k_var).grid(row=1, column=1)

        tk.Label(self, text="Duty Cycle Length (s):").grid(row=2, column=0)
        self.cycle_var = tk.IntVar(value=300)
        tk.Entry(self, textvariable=self.cycle_var).grid(row=2, column=1)

        tk.Label(self, text="Buffer Capacity (??):").grid(row=3, column=0)
        self.buffer_var = tk.StringVar(value="inf")
        tk.Entry(self, textvariable=self.buffer_var).grid(row=3, column=1)
        
        tk.Label(self, text="Bandwidth (??):").grid(row=4, column=0)
        self.bandwidth_var = tk.StringVar(value="inf")
        tk.Entry(self, textvariable=self.bandwidth_var).grid(row=4, column=1)

        

    def getOptions(self):
        """Return a   dictionary of node options"""
        param = {}
        param["epoch"] = self.epoch_var.get()
        param["k"] = self.k_var.get()
        param["tick_rate"] = self.cycle_var.get()
        param["buffer_capacity"] = float(self.buffer_var.get())
        param["bandwidth"] = float(self.bandwidth_var.get())
        
        return param

    def setOptions(self, options):
        self.epoch_var.set(options["epoch"])
        self.k_var.set(options["k"])
        self.cycle_var.set(options["tick_rate"])
        self.buffer_var.set(options["buffer_capacity"])
        self.bandwidth_var.set(options["bandwidth"])


class NodeChoices(tk.LabelFrame):
    def __init__(self, parent):
        tk.LabelFrame.__init__(self, parent, text="NodeChoices")
        self.parent = parent

        self.node_vars = []

        for (i, node) in zip(range(len(NODES)), NODES):
            v = tk.IntVar()
            self.node_vars.append(v)
            tk.Checkbutton(self, text=node.__name__, variable=v).pack(anchor=W)

    def getChoices(self):
        """Return a   dictionary of node choices"""
        param = {}
        param["Nodes"] = []
        for (node, var) in zip(NODES, self.node_vars):
            if var.get() == 1:
                param["Nodes"].append(node)
        
        return param

    def setChoices(self, options):
        # reset 
        for var in self.node_vars:
            var.set(0)

        for (node, var) in zip(NODES, self.node_vars):
            if node in options["Nodes"]:
                var.set(1)


class TrafficOptions(tk.LabelFrame):
    def __init__(self, parent):
        tk.LabelFrame.__init__(self, parent, text="TrafficOptions")
        self.parent = parent

        tk.Label(self, text="Packet Generation Interval (s):").grid(row=0, column=0)
        self.packet_interval_var = tk.IntVar(value=300)
        tk.Entry(self, textvariable=self.packet_interval_var).grid(row=0, column=1)

        tk.Label(self, text="Packets Generated per Interval:").grid(row=1, column=0)
        self.packet_gen_var = tk.IntVar(value=1)
        tk.Entry(self, textvariable=self.packet_gen_var).grid(row=1, column=1)

        tk.Label(self, text="Time to Live (s):").grid(row=2, column=0)
        self.ttl_var = tk.StringVar(value=0)
        tk.Entry(self, textvariable=self.ttl_var).grid(row=2, column=1)

        tk.Label(self, text="Packet Size (??):").grid(row=3, column=0)
        self.packet_size_var = tk.IntVar(value=0)
        tk.Entry(self, textvariable=self.packet_size_var).grid(row=3, column=1)

    def getOptions(self):
        """Return a   dictionary of traffic options"""
        param = {}
        param["step"] = self.packet_interval_var.get()
        param["traffic_per_step"] = self.packet_gen_var.get()
        param["time_to_live"] = float(self.ttl_var.get())
        param["payload"] = self.packet_size_var.get()

        return param

    def setOptions(self, options):
        self.packet_interval_var.set(options["step"])
        self.packet_gen_var.set(options["traffic_per_step"])
        self.ttl_var.set(options["time_to_live"])
        self.packet_size_var.set(options["payload"])


class GraphingOptions(tk.LabelFrame):
    def __init__(self, parent):
        tk.LabelFrame.__init__(self, parent, text="GraphicsOptions")
        self.parent = parent

        frame1 = tk.Frame(self)
        frame1.pack(side=tk.LEFT)
        frame2 = tk.Frame(self)
        frame2.pack(side=tk.LEFT)

        self.ind_input = tk.StringVar()
        tk.Entry(frame1, textvariable=self.ind_input).pack()

        self.ind_choice = tk.StringVar()
        self.ind_choice.set("None")
        for choice in INDEPENDANT:
            tk.Radiobutton(frame1, text=choice, variable=self.ind_choice, value=choice).pack(anchor=W)

        self.dep_vars = []

        for dep in DEPENDANT:
            v = tk.IntVar()
            self.dep_vars.append(v)
            tk.Checkbutton(frame2, text=dep, variable=v).pack(anchor=W)


        


    def getOptions(self):
        """Return a dictionary of graphing (output) options"""
        param = {}
        param["choice"] = self.ind_choice.get()
        param["input"] = self.ind_input.get()
        if param["choice"] != "None":
            param["input"] = [
                input.strip()
                for input in self.ind_input.get().split(",")
                if input != ""
            ]
            if param["choice"] != "contact_dir":
                param["input"] = [
                    int(input)
                    for input in param["input"]
                ]
        param["dependant"] = []
        for (dep, var) in zip(DEPENDANT, self.dep_vars):
            if var.get() == 1:
                param["dependant"].append(dep)
        
        
        return param

    def setOptions(self, options):
        self.ind_choice.set(options["choice"])
        tmp = ", ".join(str(i) for i in options["input"])
        self.ind_input.set(tmp)

        for var in self.dep_vars:
            var.set(0)

        for (dep, var) in zip(DEPENDANT, self.dep_vars):
            if dep in options["dependant"]:
                var.set(1)


class Gui(tk.Tk):
    def __init__(self, parent):
        tk.Tk.__init__(self, parent)
        self.parent = parent

        self.param={}

        self.simOptions = SimOptions(self)
        self.nodeChoices = NodeChoices(self)
        self.nodeOptions = NodeOptions(self)
        self.trafficOptions = TrafficOptions(self)
        self.graphingOptions = GraphingOptions(self)

        self.simOptions.grid(row=0, column=0, sticky="NSEW")
        self.nodeChoices.grid(row=0, column=1, sticky="NSEW")
        self.nodeOptions.grid(row=1, column=1, sticky="NSEW")
        self.trafficOptions.grid(row=1, column=0, sticky="NSEW")
        self.graphingOptions.grid(row=0, column=2, rowspan=2, sticky="NSEW")

        tk.Button(
            self, text="Load config file", command=self.load_config
        ).grid(row=3, column=0, columnspan=3)
        tk.Button(
            self, text="Save sim parameters", command=self.submit
        ).grid(row=4, column=0, columnspan=3)
        self.runButton = tk.Button(
            self, text="Save and run sim", command=self.submitQuit
        )
        self.runButton.grid(row=5, column=0, columnspan=3)
        tk.Button(self, text="Quit", command=self.quit).grid(
            row=6, column=0, columnspan=3
        )

    def read_config(self, file):
        with open(file, "r", newline="") as f:
            param = yaml.load(f, Loader=Loader)

        self.simOptions.setOptions(param["SimOptions"])
        self.nodeOptions.setOptions(param["NodeOptions"])
        self.nodeChoices.setChoices(param["NodeChoices"])
        self.trafficOptions.setOptions(param["TrafficOptions"])
        self.graphingOptions.setOptions(param["GraphingOptions"])

    def load_config(self):
        file = fd.askopenfilename()
        self.read_config(file)


    def submit(self):
        param = {}
        param["SimOptions"] = self.simOptions.getOptions()
        param["NodeOptions"] = self.nodeOptions.getOptions()
        param["NodeChoices"] = self.nodeChoices.getChoices()
        param["TrafficOptions"] = self.trafficOptions.getOptions()
        param["GraphingOptions"] = self.graphingOptions.getOptions()

        self.param = param
        
        with open(param["SimOptions"]["config"], "w", newline="") as f:
            yaml.dump(param, f)
        

    def submitQuit(self):
        self.submit()
        self.quit()


def main():
    gui = Gui(None)
    gui.title("Pydtnsim Gui Test")

    gui.runButton["state"] = DISABLED  
    gui.mainloop()

    print(gui.param)

if __name__ == "__main__":
    sys.exit(main())
